@@ -8,7 +8,8 @@ enum mdbox_sync_flags {
 	MDBOX_SYNC_FLAG_FORCE		= 0x01,
 	MDBOX_SYNC_FLAG_FSYNC		= 0x02,
 	MDBOX_SYNC_FLAG_FORCE_REBUILD	= 0x04,
-	MDBOX_SYNC_FLAG_NO_PURGE	= 0x08
+	MDBOX_SYNC_FLAG_NO_PURGE	= 0x08,
+	MDBOX_SYNC_FLAG_NO_REBUILD	= 0x10
 };
 
 struct mdbox_sync_context {
@@ -21,6 +22,7 @@ struct mdbox_sync_context {
 	enum mdbox_sync_flags flags;
 
 	ARRAY_TYPE(seq_range) expunged_seqs;
+	unsigned int expunged_count;
 };
 
 int mdbox_sync_begin(struct mdbox_mailbox *mbox, enum mdbox_sync_flags flags,
